package listasDoblementeEnlazadas.modelo;

/**
 * 
 * @author Lista doblemente enlazada de nodos
 * @param <T> tipo de datos de los nodos
 */
public class ListaDoblementeEnlazada<T> {
	/**
	 * Primer elemento de la lista
	 */
	private Nodo<T> cabeza;
	/**
	 * Total de elementos de la lista
	 */
	private int tamanio;

	/**
	 * Constructor por defecto
	 */
	public ListaDoblementeEnlazada() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Devuelve el tamaño de la lista
	 * 
	 * @return tamaño de la lista
	 */
	public int getTamanio() {
		return tamanio;
	}

	/**
	 * Consulta si la lista está vacia o o
	 * 
	 * @return true cuando esta vacia, false cuando no
	 */
	public boolean esVacia() {
		if (cabeza == null)
			return true;
		else
			return false;
	}

	/**
	 * Agrega un nuevo nodo al final de la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 */
	public void agregar(T valor) {
		Nodo<T> nuevo = new Nodo<T>();
		nuevo.setValor(valor);
		if (esVacia()) {
			cabeza = nuevo;
		} else {
			// agregar al final de la lista
			Nodo<T> aux = cabeza;
			while (aux.getSiguiente() != null) {
				aux = aux.getSiguiente();
			}
			aux.setSiguiente(nuevo);
			nuevo.setAnterior(aux);

		}
		tamanio++;
	}

	/**
	 * inserta un nuevo nodo en la lista
	 * 
	 * @param valor
	 *            valor a agregar
	 * @param pos
	 *            posicion donde se insertara el nodo
	 * @throws PosicionIlegalException
	 *             excepcion en caso que la posicion no exista
	 */
	public void insertar(T valor, int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos <= tamanio) {

			Nodo<T> nuevo = new Nodo<T>();
			nuevo.setValor(valor);
			// el nodo a insertar esta al comienzo de la lista
			if (pos == 0) {
				nuevo.setSiguiente(cabeza);
				cabeza = nuevo;
			} else {
				// El nodo a insertar va al final de la lista
				if (pos == tamanio) {
					Nodo<T> aux = cabeza;
					while (aux.getSiguiente() != null) {
						aux = aux.getSiguiente();
					}
					aux.setSiguiente(nuevo);
					nuevo.setAnterior(aux);
				}
				// el nodo a insertar esta en medio
				else {
					Nodo<T> aux = cabeza;
					for (int i = 0; i <= pos - 2; i++) {
						aux = aux.getSiguiente();
					}
					Nodo<T> siguiente = aux.getSiguiente();
					aux.setSiguiente(nuevo);
					nuevo.setSiguiente(siguiente);
					nuevo.setAnterior(aux);
					siguiente.setAnterior(nuevo);
				}
			}
			tamanio++;
		} else {
			throw new PosicionIlegalException();
		}

	}

	/**
	 * Devuelve el valor de una determinada posicion
	 * 
	 * @param pos
	 *            posicion
	 * @return el valor de tipo T
	 * @throws PosicionIlegalException
	 */
	public T getValor(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			T valor;
			if (pos == 0) {
				valor = cabeza.getValor();
				return valor;
			} else {
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 1; i++) {
					aux = aux.getSiguiente();
				}
				valor = aux.getValor();
			}
			return valor;
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Elimina un nodo en una determinada posicion
	 * 
	 * @param pos
	 *            posicionSystem.out.println("");
	 * @throws PosicionIlegalException
	 */
	public void remover(int pos) throws PosicionIlegalException {
		if (pos >= 0 && pos < tamanio) {
			if (pos == 0) {
				// El nodo a eliminar esta en la primera posicion
				cabeza = cabeza.getSiguiente();
				cabeza.setAnterior(null);
				tamanio--;
			} else {
				Nodo<T> aux = cabeza;
				for (int i = 0; i <= pos - 2; i++) {
					aux = aux.getSiguiente();
				}
				Nodo<T> prox = aux.getSiguiente();
				prox.getSiguiente().setAnterior(aux);
				aux.setSiguiente(prox.getSiguiente());
				// prox.setAnterior(null); //??
				// prox.setSiguiente(null); //??
				tamanio--;
			}
		} else {
			throw new PosicionIlegalException();
		}
	}

	/**
	 * Clear, elimina todos los nodos de la lista
	 */
	public void limpiar() {
		cabeza = null;
		tamanio = 0;
	}

	/**
	 * Devuelve todos los elementos de la lista
	 * 
	 * @return array de objetos
	 */
	public Object[] imprimir() {
		Nodo<T> temp = cabeza;
		Object[] nodos = new Object[tamanio];
		int i = 0;
		while (temp != null) {
			nodos[i] = temp.getValor();
			temp = temp.getSiguiente();
			i++;
		}
		return nodos;
	}

	/**
	 * Imprime por consola todos los elementos de la lista de adelante hacia
	 * atras y de atras hacia adelante. Esto sirve para probar que lista esté
	 * correctamente implementada
	 */
	public void recorrer() {
		Nodo<T> aux = cabeza;
		do {
			// Procesar el nodo
			// System.out.println(aux.getValor().toString());
			aux = aux.getSiguiente();
		} while (aux != cabeza);
	}
	/**
	 * Imprime todos los nodos, sirve para probar que funciona bien
	 */

	public void imprimirTodo() {
		Nodo<T> aux = cabeza;
		Nodo<T> ultimo = null;// Almacena la referencia del último nodo de la
								// lista
		System.out.println("Listado de todos los nodos:");
		Object nodo = new Object();
		while (aux != null) {
			ultimo = aux;
			nodo = aux.getValor();
			System.out.println("" + nodo.toString());
			aux = aux.getSiguiente();
		}
		System.out.println("De atras para adelante:");
		// De atras para adelante
		aux = ultimo;
		while (aux != null) {
			nodo = aux.getValor();
			System.out.println("" + nodo.toString());
			aux = aux.getAnterior();
		}
	}
}